@@ -31,4 +31,4 @@ timer.timeout.connect(update)
 timer.start(50)
 
 if __name__ == '__main__':
-    pg.mkQApp().exec_()
+    pg.exec()
