@@ -30,4 +30,4 @@ y = pg.pseudoScatter(vals, spacing=0.15)
 plt2.plot(vals, y, pen=None, symbol='o', symbolSize=5, symbolPen=(255,255,255,200), symbolBrush=(0,0,255,150))
 
 if __name__ == '__main__':
-    pg.mkQApp().exec_()
+    pg.exec()
