@@ -1,8 +1,8 @@
 from collections import OrderedDict
 from argparse import Namespace
 
-
-examples = OrderedDict([
+# Avoid clash with module name
+examples_ = OrderedDict([
     ('Command-line usage', 'CLIexample.py'),
     ('Basic Plotting', Namespace(filename='Plotting.py', recommended=True)),
     ('ImageView', 'ImageView.py'),
@@ -12,7 +12,7 @@ examples = OrderedDict([
     ('Plot Customization', 'customPlot.py'),
     ('Timestamps on x axis', 'DateAxisItem.py'),
     ('Image Analysis', 'imageAnalysis.py'),
-    ('Color Maps', 'colorMaps.py'),
+    ('Matrix Display', 'MatrixDisplayExample.py'),
     ('ViewBox Features', Namespace(filename='ViewBoxFeatures.py', recommended=True)),
     ('Dock widgets', 'dockarea.py'),
     ('Console', 'ConsoleWidget.py'),
@@ -29,9 +29,15 @@ examples = OrderedDict([
         ('Verlet chain', 'verlet_chain_demo.py'),
         ('Koch Fractal', 'fractal.py'),
     ])),
+    ('Colors', OrderedDict([
+        ('Color Maps', 'colorMaps.py'),
+        ('Color Map Linearization', 'colorMapsLinearized.py'),
+        ('Color Gradient Plots', 'ColorGradientPlots.py')
+    ])),
     ('GraphicsItems', OrderedDict([
         ('Scatter Plot', 'ScatterPlot.py'),
         #('PlotItem', 'PlotItem.py'),
+        ('InfiniteLine', 'InfiniteLine.py'),
         ('IsocurveItem', 'isocurve.py'),
         ('GraphItem', 'GraphItem.py'),
         ('ErrorBarItem', 'ErrorBarItem.py'),
@@ -67,6 +73,10 @@ examples = OrderedDict([
         ('Line Plot', 'GLLinePlotItem.py'),
         ('Mesh', 'GLMeshItem.py'),
         ('Image', 'GLImageItem.py'),
+        ('Text', 'GLTextItem.py'),
+        ('BarGraph', 'GLBarGraphItem.py'),
+        ('Painter', 'GLPainterItem.py'),
+        ('Gradient Legend', 'GLGradientLegendItem.py')
     ])),
     ('Widgets', OrderedDict([
         ('PlotWidget', 'PlotWidget.py'),
@@ -96,10 +106,8 @@ others = dict([
     ('MultiplePlotAxes', 'MultiplePlotAxes.py'),
     ('ROItypes', 'ROItypes.py'),
     ('ScaleBar', 'ScaleBar.py'),
-    ('InfiniteLine', 'InfiniteLine.py'),
     ('ViewBox', 'ViewBox.py'),
     ('GradientEditor', 'GradientEditor.py'),
-    ('GLBarGraphItem', 'GLBarGraphItem.py'),
     ('GLViewWidget', 'GLViewWidget.py'),
     ('DiffTreeWidget', 'DiffTreeWidget.py'),
     ('MultiPlotWidget', 'MultiPlotWidget.py'),
